import os
import sys
from pathlib import Path

from datetime import datetime
import whisper

import os
import openai

from dotenv import load_dotenv

load_dotenv()

openai.api_key = os.getenv("OPENAI_API_KEY")

user_profile = os.environ['USERPROFILE']
SEP = os.path.sep


def summarize_txt(txt):

    response = openai.Completion.create(
        model="text-davinci-003",
        prompt="Summarize the following text with one paragraph having not more than 50 words:\n\n" + txt,
        temperature=0,
        max_tokens=64,
        top_p=1.0,
        frequency_penalty=0.0,
        presence_penalty=0.0
    )

    return response.choices[0].text


def extract_txt(audio_path):
    # Transcribe the audio using OpenAI Whisper API
    model_opts = [
        "tiny",
        "base",
        "small",
        "medium",
        "large"
    ]

    model = whisper.load_model(
        name=model_opts[2]
    )
    result = model.transcribe(audio_path, verbose=False)
    txt = result['text']
    return txt


def save_transcription(audio_path, txt, summarize = False):
    
    if summarize:
        txt = summarize_txt(txt)
    
    # Save the transcription
    file_name = audio_path.split(SEP)[-1].split(".")[0] + ".txt"
    
    with open(file_name, "w", encoding="utf-8") as f:
        f.write(txt)
        
    return file_name


def main():
    """This method extract the audio from a video
    The transcriptions API takes as input the audio file you want to transcribe and the desired output file format 
    for the transcription of the audio. They currently support multiple input and output file formats.
    """

    audio_path = sys.argv[1] or input("Enter the audio file path: ")
    txt = extract_txt(audio_path)
    file_name = save_transcription(audio_path, txt)
    
    print(
        f"---------------------------------------------------------------------------------------------------------\n"
        f"Transcribed: {audio_path} to {file_name}"
        f"\n---------------------------------------------------------------------------------------------------------\n"
    )


if __name__ == '__main__':
    main()



from sys import argv
import cv2
import mediapipe as mp
from math import hypot
from ctypes import cast, POINTER
from comtypes import CLSCTX_ALL
from pycaw.pycaw import AudioUtilities, IAudioEndpointVolume
import numpy as np

# If there is an argument in the command line pass it to the function set DEBUG to False
DEBUG = argv[1] == "--debug" if len(argv) > 1 else False

cap = cv2.VideoCapture(0)  # Checks for camera
cap.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)  # Sets the width
cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)  # Sets the height
cap.set(cv2.CAP_PROP_FPS, 30)  # Sets the fps

mp_hands = mp.solutions.hands  # detects hand/finger
hands = mp_hands.Hands()  # complete the initialization configuration of hands
mpDraw = mp.solutions.drawing_utils

# To access speaker through the library pycaw
devices = AudioUtilities.GetSpeakers()
interface = devices.Activate(IAudioEndpointVolume._iid_, CLSCTX_ALL, None)
volume = cast(interface, POINTER(IAudioEndpointVolume))
volbar = 400
volper = 0

# volume range
volMin, volMax = volume.GetVolumeRange()[:2]

# while the capture device is open run the loop
while cap.isOpened():

    success, frame = cap.read()  # If camera works capture an image
    frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)  # Convert to RGB
    # flip the image
    frame = cv2.flip(frame, 1)

    # Collection of gesture information
    results = hands.process(frame)  # completes the image processing.

    lmList = []  # empty list
    if results.multi_hand_landmarks:  # list of all hands detected.
        
        # By accessing the list, we can get the information of each hand's corresponding flag bit
        for hand_landmarks in results.multi_hand_landmarks:
            
            # adding counter and returning it
            for id, lm in enumerate(hand_landmarks.landmark):
                # Get finger joint points
                h, w, _ = frame.shape
                cx, cy = int(lm.x * w), int(lm.y * h)
                # adding to the empty list 'lmList'
                lmList.append([id, cx, cy])
            
            if DEBUG:
                print(lmList)        
                mpDraw.draw_landmarks(frame, hand_landmarks, mp_hands.HAND_CONNECTIONS)

    if lmList != []:
        
        # if the index finger and middle finger are in the middle left of the screen
        # increase the volume
        # getting the value at a point as x, y
        x1, y1 = lmList[4][1], lmList[4][2]  # thumb
        x2, y2 = lmList[8][1], lmList[8][2]  # index finger

        if x1 < frame.shape[1] // 2 and x2 < frame.shape[1] // 2 and y1 < frame.shape[0] // 2 and y2 < frame.shape[0] // 2:

            # creating circle at the tips of thumb and index finger
            # image #fingers #radius #rgb
            cv2.circle(frame, (x1, y1), 13, (255, 0, 0), cv2.FILLED)
            # image #fingers #radius #rgb
            cv2.circle(frame, (x2, y2), 13, (255, 0, 0), cv2.FILLED)
            # create a line b/w tips of index finger and thumb
            cv2.line(frame, (x1, y1), (x2, y2), (255, 0, 0), 3)

            length = hypot(x2 - x1, y2 - y1)  
            # distance b/w tips using hypotenuse
            # from numpy we find our length by converting hand range in terms of volume range ie b/w -63.5 to 0
            vol = np.interp(length, [30, 350], [volMin, volMax])
            volbar = np.interp(length, [30, 350], [400, 150])
            volper = np.interp(length, [30, 350], [0, 100])

            volume.SetMasterVolumeLevel(vol, None)

    # Hand range 30 - 350
    # Volume range -63.5 - 0.0
    # creating volume bar for volume level
    # vid ,initial position ,ending position ,rgb ,thickness
    cv2.rectangle(frame, (50, 150), (85, 400), (0, 0, 255), 4)
    cv2.rectangle(frame, (50, int(volbar)), (85, 400),
                (0, 0, 255), cv2.FILLED)
    cv2.putText(frame, f"{int(volper)}%", (10, 40),
                cv2.FONT_ITALIC, 1, (0, 255, 98), 3)
    # tell the volume percentage ,location,font of text,length,rgb color,thickness

    cv2.imshow('Image', frame)  # Show the video
    if cv2.waitKey(1) & 0xff == ord('q'):  # By using spacebar delay will stop
        break

cap.release()  # stop cam
cv2.destroyAllWindows()  # close window



import pika
import os
import deepl

import docker
from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

class TranslationServer:
    def __init__(self, input_queue_name, output_queue_name, rabbitmq_host='localhost'):
        self.rabbitmq_host = rabbitmq_host
        self.input_queue_name = input_queue_name
        self.output_queue_name = output_queue_name
        self.api_key = os.environ.get('DEEPL_AUTH_KEY')

        self.input_channel = None
        self.output_channel = None

        self.connection = None
        self.connect()

    def connect(self):
        parameters = pika.ConnectionParameters(host=self.rabbitmq_host)
        self.connection = pika.BlockingConnection(parameters)

        self.input_channel = self.connection.channel()
        self.input_channel.queue_declare(queue=self.input_queue_name)

        self.output_channel = self.connection.channel()
        self.output_channel.queue_declare(queue=self.output_queue_name)

    def start_consuming(self):
        self.input_channel.basic_consume(queue=self.input_queue_name, on_message_callback=self.on_message,
                                        auto_ack=True)
        self.input_channel.start_consuming()

    def on_message(self, channel, method, properties, body):
        if body is None:
            self.send_message('.' * 10)
        message_text = body.decode('utf-8')
        translated_text = self.translate(message_text)
        self.send_message(translated_text)

    def translate(self, text):
        translator = deepl.Translator(self.api_key)
        translated_text = translator.translate_text(text, source_lang='EN', target_lang='NL').text
        return translated_text

    def send_message(self, message):
        self.output_channel.basic_publish(exchange='', routing_key=self.output_queue_name, body=message)
        print(message)

    def close(self):
        self.input_channel.close()
        self.output_channel.close()

        if self.connection is not None and not self.connection.is_closed:
            self.connection.close()

    def reconnect(self):
        self.close()
        self.connect()


def main():
    translator = TranslationServer('speech_to_text_stream', 'translated_text_stream')
    translator.start_consuming()

if __name__ == '__main__':
    main()



import os

import cv2
import pika
import threading

speech_data = []
translation_data = []

# define the RabbitMQ consumer class
class RabbitMQConsumer:

    def __init__(self, host, queue_name):
        self.host = host
        self.queue_name = queue_name
        self.channel = None

    def start_consuming(self):
        connection = pika.BlockingConnection(pika.ConnectionParameters(host=self.host))
        self.channel = connection.channel()
        self.channel.queue_declare(queue=self.queue_name)
        self.channel.basic_consume(queue=self.queue_name, on_message_callback=self.on_message, auto_ack=True)
        print(f'Started consuming messages from {self.queue_name} queue...')
        self.channel.start_consuming()

    def on_message(self, ch, method, properties, body):
        if self.queue_name == "speech_to_text_stream":
            speech_data.append(body.decode('utf-8'))
            
        elif self.queue_name == "translated_text_stream":
            translation_data.append(body.decode('utf-8'))


# create instance of RabbitMQ consumer class
voice_consumer = RabbitMQConsumer('localhost', 'speech_to_text_stream')

# create an instance of RabbitMQ consumer for the translation
translation_consumer = RabbitMQConsumer('localhost', 'translated_text_stream')

# start the RabbitMQ consumer thread
voice_thread = threading.Thread(target=voice_consumer.start_consuming)
voice_thread.start()

# start the RabbitMQ consumer thread
translation_thread = threading.Thread(target=translation_consumer.start_consuming)
translation_thread.start()

# start the video stream
cap = cv2.VideoCapture(0)

# change the video frame size to 1280x720
cap.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)
cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)
cap.set(cv2.CAP_PROP_FPS, 30)


while True:
    # read a frame from the video stream
    ret, frame = cap.read()

    # check if the frame is valid
    if not ret:
        break

    # display the recognized text
    if len(speech_data) > 0:
        # put the text at the bottom center of the frame and make the font size 12pt and white with border and gray background
        cv2.putText(frame, speech_data[-1], (int(frame.shape[1] / 2) - 100, frame.shape[0] - 20),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 255, 255), 1, cv2.LINE_AA)
        
        
    if len(translation_data) > 0:
        # put the text at the bottom center of the frame and make the font size 12pt and white with border and gray background
        cv2.putText(frame, translation_data[-1], (int(frame.shape[1] / 2) - 100, 20),
                    cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 255, 255), 1, cv2.LINE_AA)

    # display the video frame
    cv2.imshow('frame', frame)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

    # if c is pressed, then clear the speech data
    if cv2.waitKey(1) & 0xFF == ord('c'):
        speech_data.clear()

    # if s is pressed, then save the speech data to a text file
    if cv2.waitKey(1) & 0xFF == ord('s'):
        with open('speech.txt', 'w') as f:
            f.write(os.linesep.join(speech_data))

# release the video stream and destroy all windows
cap.release()
cv2.destroyAllWindows()

# stop the RabbitMQ consumer thread
voice_thread.join()
translation_thread.join()



from flask import Flask, render_template, Response
import cv2

app = Flask(__name__)

camera = cv2.VideoCapture(0)  # use 0 for web camera
camera.set(cv2.CAP_PROP_FRAME_WIDTH, 1280)
camera.set(cv2.CAP_PROP_FRAME_HEIGHT, 720)
camera.set(cv2.CAP_PROP_FPS, 30)

def gen_frames():  # generate frame by frame from camera
    while True:
        # Capture frame-by-frame
        success, frame = camera.read()  # read the camera frame
        if not success:
            break
        else:
            _, buffer = cv2.imencode('.jpg', frame)
            frame = buffer.tobytes()
            yield (b'--frame\r\n'
                b'Content-Type: image/jpeg\r\n\r\n' + frame + b'\r\n')  # concat frame one by one and show result


@app.route('/video_feed')
def video_feed():
    # Video streaming route. Put this in the src attribute of an img tag
    return Response(gen_frames(), mimetype='multipart/x-mixed-replace; boundary=frame')


@app.route('/')
def index():
    """Video streaming home page."""
    return render_template('index.html')

def main():
    app.run(host='0.0.0.0', port=8080)

if __name__ == '__main__':
    main()



import requests
from bs4 import BeautifulSoup
import re

# define a function to extract text from a webpage
def extract_text(url):
    response = requests.get(url)
    soup = BeautifulSoup(response.content, "html.parser")
    text = ""
    for p in soup.find_all("p"):
        text += p.get_text()
    return text

# preprocess the data
def preprocess(text):
    # remove URLs
    text = re.sub(r"http\S+", "", text)
    # remove special characters and digits
    text = re.sub(r"[^a-zA-Z\s]", "", text)
    # convert to lowercase
    text = text.lower()
    return text


def main():
    
    # collect data from news websites
    news_urls = [
        "https://www.nytimes.com/",
        "https://www.washingtonpost.com/",
        "https://www.bbc.com/news",
    ]
    news_text = []
    for url in news_urls:
        news_text.append(extract_text(url))
        
        
    # collect data from social media platforms
    social_media_urls = [
        "https://www.twitter.com/",
        "https://www.facebook.com/",
        "https://www.instagram.com/",
    ]
    social_media_text = []
    for url in social_media_urls:
        social_media_text.append(extract_text(url))
        
        
    # collect data from blogs
    blog_urls = [
        "https://www.medium.com/",
        "https://www.wordpress.com/",
        "https://www.blogger.com/",
    ]
    blog_text = []
    for url in blog_urls:
        blog_text.append(extract_text(url))
    
    news_text = [preprocess(text) for text in news_text]
    social_media_text = [preprocess(text) for text in social_media_text]
    blog_text = [preprocess(text) for text in blog_text]

    # combine the datasets
    training_data = news_text + social_media_text + blog_text
    
    print(training_data)
    
    
if __name__ == "__main__":
    main()


from transformers import pipeline

def sentiment_analysis(text):

    # load the sentiment analysis model
    classifier = pipeline("sentiment-analysis")

    # apply sentiment analysis
    result = classifier(text)
    
    return result


def main():
    
    # sample text
    text = "I love this new phone, it's amazing!"
    
    result = sentiment_analysis(text)

    # print the results
    print("Original text:", text)
    print("Sentiment analysis result:", result)
    
    
if __name__ == "__main__":
    main()


import cv2
from cvzone.HandTrackingModule import HandDetector
from nltk import decorator
from pynput.keyboard import Controller
from screeninfo import get_monitors

from datetime import datetime
from typing import Optional
import os
import threading
import requests

import pandas as pd
import numpy as np

import cv2
import mediapipe as mp

from gtts import gTTS
from pygame import mixer

from newsapi import NewsApiClient

import docker
from dotenv import load_dotenv


load_dotenv()  # take environment variables from .env.

# Constants and global variables
api = NewsApiClient(api_key=os.environ['NEWSAPI_ORG'])
user_profile = os.environ['USERPROFILE']
SEP = os.path.sep
LOGO_URL = "https://raw.githubusercontent.com/codesapienbe/aij-webcam/master/logo.png"
CSV_URL = "https://raw.githubusercontent.com/codesapienbe/aij-webcam/master/news.csv"

SCREEN_WIDTH = 1280
SCREEN_HEIGHT = 720
SCREEN_FPS = 60
SCREEN_START_X = 0
SCREEN_START_Y = 0
SCREEN_MAXIMIZE = True
APP_TITLE = 'AIJ'


start = datetime.now().strftime("%Y%m%d%H%M%S")
start_dt = datetime.now()
config_home = user_profile + SEP + '.aij'
config_abs = config_home + SEP + 'config.json'
image_home = user_profile + SEP + '.aij' + SEP + 'image'
logo_abs = image_home + SEP + 'logo.png'
csv_home = user_profile + SEP + '.aij' + SEP + 'news'
csv_abs = csv_home + SEP + 'news.csv'
audio_home = user_profile + SEP + '.aij' + SEP + 'audio'
screenshot_home = user_profile + SEP + '.aij' + SEP + 'screenshot'
audio_abs = audio_home + SEP + start + '.mp3'
video_home = user_profile + SEP + '.aij' + SEP + 'video'
news_df = Optional[pd.DataFrame]
weather_df = Optional[pd.DataFrame]


def pre_init():
    """
    This function is called before the server is initialized
    """
    client = docker.from_env()

    # print all containers
    containers = client.containers.list()

    for container in containers:
        print(
            container.name,
            container.id,
            container.status
        )

    # if there is no docker container running named aij-messaging-server then run the server script 'aijinit'
    if not any(container.name == 'aij-messaging-server' for container in containers):
        os.system('aijinit')


def get_weather():
    """
    Get the weather from the weatherapi.com API
    """
    api_key = os.environ['WEATHERAPI_COM']
    region = os.environ['WEATHERAPI_COM_REGION']
    response = requests.get(
        f'http://api.weatherapi.com/v1/current.json?key={api_key}&q={region}&aqi=no'
    )
    return response.json()


def get_weather_temp_c(weather):
    """
    Get the temperature in Celsius
    """
    return weather['current']['temp_c']


def get_weather_temp_f(weather):
    """
    Get the temperature in Fahrenheit
    """
    return weather['current']['temp_f']


def get_weather_to_df(weather):
    """
    Convert the weather to a dataframe
    """
    # map all the articles to a list
    current = weather['current']

    # create a dataframe
    df = pd.DataFrame(current, columns=['temp_c', 'temp_f', 'condition', 'wind_mph', 'wind_kph', 'wind_degree', 'wind_dir', 'pressure_mb', 'pressure_in',
                      'precip_mm', 'precip_in', 'humidity', 'cloud', 'feelslike_c', 'feelslike_f', 'vis_km', 'vis_miles', 'uv', 'gust_mph', 'gust_kph'])

    return df


def get_top_headlines():
    """
    Get the top headlines from the newsapi.org API
    """
    response = api.get_top_headlines(
        sources='bbc-news, cnn, fox-news, google-news, the-new-york-times, the-wall-street-journal, the-washington-post, time, usa-today, wired'
    )
    return response


def news_to_df(news):
    """
    Convert the news to a dataframe
    """
    # map all the articles to a list
    articles = list(map(lambda x: x['title'], news['articles']))

    # create a dataframe
    df = pd.DataFrame(articles, columns=['title'])

    # add a column for the length of the title
    df['title_length'] = df['title'].apply(lambda x: len(x))

    # add a column for the number of words in the title
    df['title_words'] = df['title'].apply(lambda x: len(x.split(' ')))

    return df


def download_logo():
    """
    Download the logo.png file to userprofile/.aij/images/logo.png
    """
    logo_remote_response = requests.get(
        LOGO_URL, allow_redirects=True, stream=True, timeout=10)

    if logo_remote_response.status_code == 200:
        # save the logo.png file if the response is ok and the file does not exist
        if not os.path.exists(logo_abs):
            with open(logo_abs, 'wb') as logo_file:
                logo_file.write(logo_remote_response.content)


def download_news_csv() -> None:
    """
    Download the news.csv file to userprofile/.aij/news/news.csv
    """
    csv_remote_response = requests.get(
        CSV_URL, allow_redirects=True, stream=True, timeout=10)

    if csv_remote_response.status_code == 200:
        # save the news.csv file if the response is ok and the file does not exist
        if not os.path.exists(csv_abs):
            with open(csv_abs, 'wb') as csv_file:
                csv_file.write(csv_remote_response.content)


def make_directories(paths: list):
    """
    Create directories if not exists
    """
    for path in paths:
        if not os.path.exists(path):
            os.mkdir(path)


class Label:
    def __init__(self, x, y, w, h, v, s):
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.v = v
        self.s = s

    def draw(self, img):
        cv2.putText(img, self.v, (self.x + 25, self.y + 40),
                    cv2.FONT_HERSHEY_PLAIN, self.s, (255, 255, 255), self.s)

    def resize(self, w, h, img):
        cv2.putText(img, self.v, (self.x + w, self.y + h),
                    cv2.FONT_HERSHEY_PLAIN, self.s, (255, 255, 255), self.s)

    def color(self, rgb, img):
        cv2.putText(img, self.v, (self.x + 25, self.y + 40),
                    cv2.FONT_HERSHEY_PLAIN, self.s, rgb, self.s)

    def text(self, value, img):
        cv2.putText(img, value, (self.x + 25, self.y + 40),
                    cv2.FONT_HERSHEY_PLAIN, self.s, (255, 255, 255), self.s)

    def shrink(self, value, limit, img):
        cv2.putText(img, (self.v[:limit + 1][-1] + value), (self.x + 25, self.y + 50), cv2.FONT_HERSHEY_PLAIN, 2,
                    (255, 255, 255),
                    2)


class Rectangle:

    def __init__(self, x, y, w, h):
        self.x = x
        self.y = y
        self.w = w
        self.h = h

    def draw(self, img):
        self.background((255, 255, 255), img)
        self.border((255, 255, 255), img)

    def background(self, rgb, img):
        cv2.rectangle(img, (self.x, self.y), (self.x + self.w,
                                              self.y + self.h), rgb, cv2.FILLED)

    def border(self, rgb, img):
        cv2.rectangle(img, (self.x, self.y),
                      (self.x + self.w, self.y + self.h), rgb, 3)


class Button:

    def __init__(self, pos, width, height, value):
        self.pos = pos
        self.width = width
        self.height = height
        self.value = value

    def click(self, img, x, y):
        if self.pos[0] < x < self.pos[0] + self.width and self.pos[1] < y < self.pos[1] + self.height:
            self.border((255, 255, 255), img)
            self.text(self.value, img)
            return True

        else:
            return False

    def draw(self, img):
        self.border((255, 255, 255), img)
        self.text(self.value, img)

    def background(self, rgb, img):
        # for the background calculator
        cv2.rectangle(
            img, self.pos, (self.pos[0] + self.width, self.pos[1] + self.height), rgb, cv2.FILLED)

    def border(self, rgb, img):
        # for the border calculator
        cv2.rectangle(
            img, self.pos, (self.pos[0] + self.width, self.pos[1] + self.height), rgb, 3)

    def text(self, value, img):
        self.value = value
        cv2.putText(img, self.value, (self.pos[0] + 25, self.pos[1] + 50), cv2.FONT_HERSHEY_PLAIN, 3, (255, 255, 255),
                    3)

    def text_hover(self, value, img):
        self.value = value
        cv2.putText(img, self.value, (self.pos[0] + 25, self.pos[1] + 50), cv2.FONT_HERSHEY_PLAIN, 6, (255, 255, 255),
                    3)

    def move(self, pos, img):
        self.pos = pos
        self.border((255, 255, 255), img)
        self.text(self.value, img)


class Keyboard():
    """
    Keyboard class: contains the keys and values, draws them
    """
    
    KEYBOARD_START_POS = (60, 20)
    KEYBOARD_KEY_SIZE = (80, 70)
    
    def __init__(self, pos = KEYBOARD_START_POS, size = KEYBOARD_KEY_SIZE):
        
        self.controller = Controller()    
        self.visible = True
        self.physical = True
        self.pos = pos
        self.size = size
        
        self.keys = [
            ["a", "o", "1"],
            ["b", "p", "2"],
            ["c", "r", "3"],
            ["d", "s", "4"],
            ["e", "t", "5"],
            ["f", "u", "6"],
            ["g", "v", "7"],
            ["h", "w", "8"],
            ["i", "x", "9"],
            ["j", "y", "0"],
            ["l", "z", "."],
            ["m", "/", ","],
        ]
        
        self.values = []
        
        self.current = ""
        self.prev = ""
        self.next = ""
        
        self.fill()
        
    def is_visible(self):
        return self.visible
    
    def is_physical(self):
        return self.physical
    
    def set_visible(self, visible):
        self.visible = visible
    
    def set_physical(self, physical):
        self.physical = physical
        
    def fill(self):
    
        button_components = []
        
        # assign the buttons to the keyboard
        # length of the first array refers to the number of rows
        # length of the second array refers to the number of columns
        rows = len(self.keys)
        cols = len(self.keys[0])
        for x in range(rows):
            for y in range(cols):
                pos_x = x * self.size[0] + self.pos[0]
                pos_y = y * self.size[1] + 100 + self.pos[1]
                button_components.append(
                    Button(
                        (pos_x, pos_y), self.size[0], self.size[1], self.keys[x][y]
                    )
                )
                
                
        self.values = button_components
        
    def clear(self):
        self.values = []
        self.current = ""
        self.prev = ""
        self.next = ""
        
    def move(self, pos):
        self.pos = pos
        
    def resize(self, size):
        self.size = size
        

class NewsAPISearch():
    """
    This class provides functions to search for news
    @param: query
    @return: Search results as a list
    """
    def __init__(self, query):
        self.query = query
    

def draw_text(image, x, y, text, color):
    """
    Draw text on the image
    """
    # draw a black rectangle on the image
    cv2.rectangle(
        image,
        (x, y - 25),
        # until the end of screen width
        (x + image.shape[1], y),
        (0, 0, 0),
        -1
    )

    # draw the text on the image
    cv2.putText(
        image,
        text,
        (x, y),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size / 12,
        color,
        2,
        cv2.LINE_AA
    )


def draw_box(image, x, y, color):
    """
    Draw a box on the image
    """
    cv2.rectangle(
        image,
        (x, y),
        (x + box_size, y + box_size),
        color,
        2
    )


def draw_button(image, x, y, text, color):
    """
    Draw a button on the image and write text on it
    """
    cv2.rectangle(
        image,
        (x, y),
        (x + box_size, y + box_size),
        color,
        2
    )

    cv2.putText(
        image,
        text,
        (x + 5, y + 30),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size,
        color,
        2,
        cv2.LINE_AA
    )


def draw_zoomed_text(image, x, y, text, color):
    """
    Draw zoomed text on the image
    """
    cv2.putText(
        image,
        text,
        (x, y),
        cv2.FONT_HERSHEY_SIMPLEX,
        font_size,
        color,
        2,
        cv2.LINE_AA
    )


zoomed_text_color = (0, 255, 0)
standard_text_color = (255, 255, 255)
color = standard_text_color

direction = 0
font_size = 12
box_size = 50
is_news_sliding = True
is_news_playing = True


MAX_NUMBER_OF_HANDS_TO_DETECT = 2

def generate_loading_animation(img):
    """
    Generate the loading animation with an overlay on the image
    """
    overlay = np.zeros((img.shape[0], img.shape[1], 3), np.uint8)
    
    cv2.rectangle(
        overlay,
        (0, 0),
        (img.shape[1], img.shape[0]),
        (0, 0, 0),
        -1
    )
    
    img = cv2.addWeighted(overlay, 0.5, img, 0.5, 0)
    return img

def main():
        
    # do not wait for the directories to be created
    thread_download_logo = threading.Thread(target=download_logo)
    thread_download_news_csv = threading.Thread(target=download_news_csv)
    thread_download_logo.start()
    thread_download_news_csv.start()
    
    make_directories([config_home, image_home, csv_home,
                    audio_home, screenshot_home, video_home])

    # if the internet connection is not available then use the local news.csv file
    network_status = os.system('ping -n 1 www.google.com')
    if network_status == 0:
        # get the top headlines from the newsapi.org API
        news = get_top_headlines()
        # convert the news to a dataframe
        news_df = news_to_df(news)

        # get the weather from the weatherapi.com API
        weather = get_weather()
        # convert the weather to a dataframe
        weather_df = get_weather_to_df(weather)

    else:
        news_df = pd.read_csv(csv_abs)
        weather_df = pd.DataFrame()
        weather_df['temp_c'] = [0]
        weather_df['temp_f'] = [0]
        weather_df['condition'] = ['']

    # initialize the video capture
    cap = cv2.VideoCapture(0)
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, SCREEN_WIDTH)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, SCREEN_HEIGHT)
    cap.set(cv2.CAP_PROP_FPS, SCREEN_FPS)

    # get the top headlines from the newsapi.org API
    titles = news_df['title'].str.cat(sep=' ') + ' ... |'
    weather = f"{weather_df['temp_c'][0]}c"
    
    keyboard = Keyboard()

    # initialize the hand detector
    detector = HandDetector(detectionCon=0.8, maxHands=MAX_NUMBER_OF_HANDS_TO_DETECT)
            
    # to avoid duplicated value inside keyboard in event writing
    delay_counter = 0

    while cap.isOpened():

        _, img = cap.read()
        img = cv2.flip(img, 1)
        # detection hand
        hand, img = detector.findHands(img, flipType=False)

        cv2.rectangle(img, (350, 30), (350 + 280, 100), (255, 255, 255), 2)

        equation_label = Label(350, 40, 400, 40, keyboard.current, 3)
        equation_label.draw(img)

        for button in keyboard.values:
            button.draw(img)

        if len(hand) == 1:
            
            landmarks = hand[0]["lmList"]
            distance, _, img = detector.findDistance(
                landmarks[8][:2], landmarks[12][:2], img)
            x, y = landmarks[8][:2]

            if distance < 70:
                for button in keyboard.values:
                    if button.click(img, x, y) and delay_counter == 0:
                        button.background((255, 255, 255), img)
                        # keyboard.press(button.value)
                        keyboard.current += button.value
                        delay_counter = 1

        elif len(hand) == 2:
            # if one hand is open and the other is closed then write with the opened hand
            left_hand = hand[0]["lmList"]
            right_hand = hand[1]["lmList"]
            
            lh = left_hand[8][1] < left_hand[9][1]
            rh = right_hand[8][1] < right_hand[9][1]
            
            if lh and not rh:
                distance, _, img = detector.findDistance(
                left_hand[8][:2], left_hand[12][:2], img)
                x, y = left_hand[8][:2]

                if distance < 70:
                    for button in keyboard.values:
                        if button.click(img, x, y) and delay_counter == 0:
                            button.background((255, 255, 255), img)
                            # keyboard.press(button.value)
                            keyboard.current += button.value
                            delay_counter = 1
                            
            elif rh and not lh:
                distance, _, img = detector.findDistance(
                right_hand[8][:2], right_hand[12][:2], img)
                x, y = right_hand[8][:2]
                
                if distance < 70:
                    for button in keyboard.values:
                        if button.click(img, x, y) and delay_counter == 0:
                            button.background((255, 255, 255), img)
                            # keyboard.press(button.value)
                            keyboard.current += button.value
                            delay_counter = 1
                            
            # if both hands are closed then make the keyboard invisible
            elif lh and rh:
                keyboard.visible = False
                img = generate_loading_animation(img)
                delay_counter = 1
            
            # if both hands are opened then show the keyboard
            elif not lh and not rh:
                keyboard.visible = True
                delay_counter = 1
            
        # avoid duplicates
        if delay_counter != 0:
            delay_counter += 1
            # i did not add value into display calculator
            # after passing 10 frames
            if delay_counter > 10:
                delay_counter = 0

        if keyboard.is_visible():
            keyboard.fill()
        else:
            keyboard.clear()

        if is_news_sliding and direction == 0:
            titles = titles[1:] + titles[0]
        elif is_news_sliding and direction == 1:
            titles = titles[-1] + titles[:-1]
        elif direction == 2:
            pass

        # draw the text
        draw_text(img, 2, img.shape[0] - 10, titles, color)

        # draw the weather on the image if the weather is available
        if weather_df['temp_c'][0] != 0:
            # set the position of the weather to the right top corner
            draw_text(img, img.shape[1] - 250, 30, weather, color)

        # if there is a logo then draw it
        if os.path.exists(logo_abs):
            logo = cv2.imread(logo_abs)
            # Resize the logo to match the height of the main image and keep the aspect ratio
            # resize the logo to 100px height and keep the aspect ratio
            logo_resized = cv2.resize(
                logo, (100, 100), interpolation=cv2.INTER_AREA)
            # get the height and width of the logo
            logo_height, logo_width, _ = logo_resized.shape
            # get the height and width of the main image
            image_height, image_width, _ = img.shape
            # calculate the x and y coordinates of the logo
            x = image_width - logo_width - 10
            y = image_height - logo_height - 30
            # draw the logo on the main image
            img[y:y + logo_height, x:x + logo_width] = logo_resized
        else:
            draw_text(img, 0, 20, 'AIJ', color)

        cv2.imshow("image", img)

        # wait for escape key
        key = cv2.waitKey(1)
        
        # if the key is escape then break
        if key == 27:
            break

    cv2.destroyAllWindows()
    cap.release()
    
    # kill all threads
    thread_download_logo.join()
    thread_download_news_csv.join()


if __name__ == "__main__":
    main()



import cv2
import mediapipe as mp

mp_drawing = mp.solutions.drawing_utils
mp_objectron = mp.solutions.objectron

# For webcam input:
cap = cv2.VideoCapture(0)

with mp_objectron.Objectron(static_image_mode=False,
                            max_num_objects=5,
                            min_detection_confidence=0.5,
                            min_tracking_confidence=0.75,
                            model_name='Cup') as objectron:
    while cap.isOpened():
        success, image = cap.read()
        if not success:
            print("Ignoring empty camera frame.")
            # If loading a video, use 'break' instead of 'continue'.
            continue

        # To improve performance, optionally mark the image as not writeable to
        # pass by reference.
        image.flags.writeable = False
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        results = objectron.process(image)

        # Draw the box landmarks on the image.
        image.flags.writeable = True
        image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
        if results.detected_objects:
            for detected_object in results.detected_objects:
                mp_drawing.draw_landmarks(
                    image, detected_object.landmarks_2d, mp_objectron.BOX_CONNECTIONS)
                mp_drawing.draw_axis(image, detected_object.rotation,
                                    detected_object.translation)
        # Flip the image horizontally for a selfie-view display.
        cv2.imshow('Objectron', cv2.flip(image, 1))

        
        # if escape or q is pressed, break the loop
        if (cv2.waitKey(5) & 0xFF == 27) or (cv2.waitKey(5) & 0xFF == ord('q')):
            break
    

cap.release()
cv2.destroyAllWindows()



import requests

# Define the claim to check
claim = "The earth is flat."

# Make a request to the FactCheck API
response = requests.get("https://factchecktools.googleapis.com/v1alpha1/claims:search", params={"query": claim})

# Check if the response contains any results
if response.json()["claims"]:
    # If there are results, print the verdict and explanation
    verdict = response.json()["claims"][0]["claimReview"][0]["textualRating"]
    explanation = response.json()["claims"][0]["claimReview"][0]["textualRatingExplanation"]
    print(f"The claim '{claim}' is {verdict}. {explanation}")
else:
    # If there are no results, print a message indicating that the claim could not be verified
    print(f"Could not verify the claim '{claim}'.")


import cv2
import numpy as np
from keras.models import load_model

# Load the trained model
model = load_model('model.h5')

# Define the emotion labels
emotions = ['Angry', 'Disgust', 'Fear', 'Happy', 'Neutral', 'Sad', 'Surprise']

# Define a function to detect the face in a frame
def detect_face(frame):
    # Convert the frame to grayscale
    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    # Load the Haar cascade for face detection
    face_cascade = cv2.CascadeClassifier('haarcascade_frontalface_default.xml')
    # Detect faces in the grayscale image
    faces = face_cascade.detectMultiScale(gray, scaleFactor=1.3, minNeighbors=5)
    # If no faces are detected, return None
    if len(faces) == 0:
        return None, None
    # If multiple faces are detected, use the largest one
    largest_face = faces[0]
    for face in faces:
        if face[2] * face[3] > largest_face[2] * largest_face[3]:
            largest_face = face
    # Extract the face region from the frame
    x, y, w, h = largest_face
    face_roi = gray[y:y+h, x:x+w]
    # Resize the face region to 48x48 pixels
    face_roi = cv2.resize(face_roi, (48, 48))
    # Return the face region and the coordinates of the face
    return face_roi, largest_face

# Define a function to predict the emotion in a face
def predict_emotion(face):
    # Reshape the face to match the input shape of the model
    face = face.reshape(1, 48, 48, 1)
    # Normalize the pixel values to be between 0 and 1
    face = face / 255.0
    # Predict the emotion label using the trained model
    predictions = model.predict(face)
    # Return the predicted emotion label
    return emotions[np.argmax(predictions)]

# Open a video stream
cap = cv2.VideoCapture(0)

while True:
    # Read a frame from the video stream
    ret, frame = cap.read()
    # Detect the face in the frame
    face, coords = detect_face(frame)
    # If a face is detected, predict the emotion label and draw a rectangle around the face
    if face is not None:
        emotion = predict_emotion(face)
        x, y, w, h = coords
        cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)
        cv2.putText(frame, emotion, (x, y-10), cv2.FONT_HERSHEY_SIMPLEX, 0.9, (0, 255, 0), 2)
    # Display the frame
    cv2.imshow('Face Emotion Recognition', frame)
    # Wait for a key press
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

# Release the video stream and close all windows
cap.release()
cv2.destroyAllWindows()


import pika
import speech_recognition as sr
import os, sys
import docker
from dotenv import load_dotenv

load_dotenv()  # take environment variables from .env.

class VoiceToTextServer:
    def __init__(self, source_lang='en-US'):
        self.source_lang = source_lang
        self.timer = 0
        self.connection = pika.BlockingConnection(
            pika.ConnectionParameters('localhost'))
        self.channel = self.connection.channel()
        self.channel.queue_declare(queue='speech_to_text_stream')
        self.r = sr.Recognizer()
        self.r.dynamic_energy_threshold = True

    def start(self):
        # start listening to mic stream
        with sr.Microphone() as source:
            self.r.adjust_for_ambient_noise(source)
            audio = self.r.listen(source, timeout=30, phrase_time_limit=5)

            try:
                recognized_text = self.r.recognize_whisper_api(
                    audio_data=audio, 
                    model="whisper-1",
                    api_key=os.getenv('WHISPER_API_KEY')
                )
                self.channel.basic_publish(
                    exchange='', routing_key='speech_to_text_stream', body=recognized_text)
                print(recognized_text)

            except sr.UnknownValueError as e:
                self.reconnect()

            except sr.RequestError as e:
                self.reconnect()

        self.start()

    def reconnect(self):
        
        # if self.connection.is_open:
        #     self.connection.close()
        # # reconnect to RabbitMQ
        # self.connection = pika.BlockingConnection(
        #     pika.ConnectionParameters('localhost'))
        # self.channel = self.connection.channel()
        self.channel.queue_declare(queue='speech_to_text_stream')
        self.channel.basic_publish(
            exchange='', routing_key='speech_to_text_stream', body='.')

    def close(self):
        print("Closing the connection to RabbitMQ...")
        if self.connection.is_open:
            self.connection.close()


def main():
    recognizer = VoiceToTextServer()
    recognizer.start()
    recognizer.close()


if __name__ == '__main__':
    main()



import cv2
import mediapipe as mp
import numpy as np

mp_drawing = mp.solutions.drawing_utils
mp_selfie_segmentation = mp.solutions.selfie_segmentation

# For static images:
IMAGE_FILES = []
BG_COLOR = (192, 192, 192)  # gray
MASK_COLOR = (255, 255, 255)  # white
with mp_selfie_segmentation.SelfieSegmentation(
        model_selection=0) as selfie_segmentation:
    for idx, file in enumerate(IMAGE_FILES):
        image = cv2.imread(file)
        image_height, image_width, _ = image.shape
        # Convert the BGR image to RGB before processing.
        results = selfie_segmentation.process(
            cv2.cvtColor(image, cv2.COLOR_BGR2RGB))

        # Draw selfie segmentation on the background image.
        # To improve segmentation around boundaries, consider applying a joint
        # bilateral filter to "results.segmentation_mask" with "image".
        condition = np.stack((results.segmentation_mask,) * 3, axis=-1) > 0.1
        # Generate solid color images for showing the output selfie segmentation mask.
        fg_image = np.zeros(image.shape, dtype=np.uint8)
        fg_image[:] = MASK_COLOR
        bg_image = np.zeros(image.shape, dtype=np.uint8)
        bg_image[:] = BG_COLOR
        output_image = np.where(condition, fg_image, bg_image)
        cv2.imwrite('/tmp/selfie_segmentation_output' +
                    str(idx) + '.png', output_image)

# For webcam input:
BG_COLOR = (192, 192, 192)  # gray
cap = cv2.VideoCapture(0)
with mp_selfie_segmentation.SelfieSegmentation(
        model_selection=1) as selfie_segmentation:
    bg_image = None
    while cap.isOpened():
        success, image = cap.read()
        if not success:
            print("Ignoring empty camera frame.")
            # If loading a video, use 'break' instead of 'continue'.
            continue

        # Flip the image horizontally for a later selfie-view display, and convert
        # the BGR image to RGB.
        image = cv2.cvtColor(cv2.flip(image, 1), cv2.COLOR_BGR2RGB)
        # To improve performance, optionally mark the image as not writeable to
        # pass by reference.
        image.flags.writeable = False
        results = selfie_segmentation.process(image)

        image.flags.writeable = True
        image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)

        # Draw selfie segmentation on the background image.
        # To improve segmentation around boundaries, consider applying a joint
        # bilateral filter to "results.segmentation_mask" with "image".
        condition = np.stack(
            (results.segmentation_mask,) * 3, axis=-1) > 0.1
        # The background can be customized.
        #   a) Load an image (with the same width and height of the input image) to
        #      be the background, e.g., bg_image = cv2.imread('/path/to/image/file')
        #   b) Blur the input image by applying image filtering, e.g.,
        #      bg_image = cv2.GaussianBlur(image,(55,55),0)
        if bg_image is None:
            bg_image = np.zeros(image.shape, dtype=np.uint8)
            bg_image[:] = BG_COLOR
        output_image = np.where(condition, image, bg_image)

        cv2.imshow('MediaPipe Selfie Segmentation', output_image)
        if cv2.waitKey(5) & 0xFF == 27:
            break

cap.release()
